import numpy as np

import pyglet
from pyglet.window import Window, key
from pyglet.gl import *
from pywavefront import visualization, Wavefront

from . import Object


class MurtWindow(Window):

    def __init__(self):
        self.cam_pos = [0,  0, -255]
        self.cam_rot = [33, -230, 0]
        self.win_size = [640, 480]
        super().__init__(width=self.win_size[0],
                         height=self.win_size[1],
                         resizable=True)
        self.pressed_keys = {}
        self.scene = []
        self.lines_set = []
        glClearColor(0.9, 0.9, 0.875, 1)
        self.alive = True
        self.zoom = 66

    def load_scene(self, file_path):
        self.scene.append(Object(file_path))

    def on_resize(self, width, height):
        self.win_size = [width, height]
        glViewport(0, 0, width, height)

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifieres):
        self.cam_rot[1] += dx
        self.cam_rot[0] -= dy

    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        self.zoom = max(20, self.zoom+scroll_y)
        self.zoom = min(120, self.zoom+scroll_y)

    def key_updates(self):
        if key.W in self.pressed_keys:
            self.cam_pos[2] += 1
        if key.S in self.pressed_keys:
            self.cam_pos[2] -= 1
        if key.A in self.pressed_keys:
            self.cam_pos[0] += 1
        if key.D in self.pressed_keys:
            self.cam_pos[0] -= 1
        if key.LSHIFT in self.pressed_keys:
            self.cam_pos[1] += 1
        if key.LCTRL in self.pressed_keys:
            self.cam_pos[1] -= 1
        if key.Z in self.pressed_keys:
            self.cam_rot[0] += 1
        if key.X in self.pressed_keys:
            self.cam_rot[0] -= 1
        if key.Q in self.pressed_keys:
            self.cam_rot[1] += 1
        if key.E in self.pressed_keys:
            self.cam_rot[1] -= 1

    def on_close(self):
        self.alive = False

    def on_key_press(self, pressed_key, modifier):
        if pressed_key == key.ESCAPE:
            self.alive = False
        self.pressed_keys[pressed_key] = True

    def on_key_release(self, pressed_key, modifier):

        if pressed_key == key.F:
            print(self.cam_pos)
            print(self.cam_rot)
            print(self.zoom)
        if pressed_key == key.R:
            self.cam_pos = [0, 0, -180]
            self.cam_rot = [90, -180, 0]
            self.zoom = 100

        try:
            del self.pressed_keys[pressed_key]
        except:
            pass

    def draw_lines(self, lines):
        points = lines['points']
        color = lines['color']
        glBegin(GL_LINE_STRIP)
        glColor4f(*color)
        for point in points:
            glVertex3f(*point)
        glEnd()
        glLineWidth(2)

    def on_draw(self):
        self.render()

    def render(self):
        self.clear()
        self.key_updates()  # update pressed keys
        # Projection
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(self.zoom, self.win_size[0] /
                       float(self.win_size[1]), .1, 1000)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        # Translate Camera
        glTranslatef(*self.cam_pos)
        glRotatef(self.cam_rot[0], 1.0, 0.0, 0.0)
        glRotatef(self.cam_rot[1], 0.0, 1.0, 0.0)
        glRotatef(self.cam_rot[2], 0.0, 0.0, 1.0)
        # Draw Scene
        glEnable(GL_DEPTH_TEST)
        for scene in self.scene:
            scene.draw()
        # Draw Lines
        for lines in self.lines_set:
            self.draw_lines(lines)

        self.flip()

    def custom_render(self):
        self.dispatch_events()
        self.render()

    def run(self):
        while self.alive:
            event = self.dispatch_events()
            self.render()



import numpy as np


def read(filename):
    triangles = []
    vertices = []
    with open(filename) as file:
        for line in file:
            components = line.strip(' \n').split(' ')
            if components[0] == "f":
                indices = list(
                    map(lambda c: int(c.split('/')[0]) - 1, components[1:]))
                for i in range(0, len(indices) - 2):
                    triangles.append(indices[i: i + 3])
            elif components[0] == "v":  #
                vertex = list(map(lambda c: float(c), components[1:]))
                vertices.append(vertex)

    return np.array(vertices), np.array(triangles)



import numpy as np
from murt.utils import objreader
from pyglet.gl import *
from pywavefront import Wavefront, visualization


class Object():
    def __init__(self, file_path):
        self.object = Wavefront(file_path, collect_faces=True)
        self.scale, self.translate, self.rotate = [
            1, 1, 1], [0, 0, 0], [0, 0, 0]
        self.vertices, self.triangles = objreader.read(file_path)

    def get_triangles(self):
        return self.vertices, self.triangles

    def resize(self, x, y, z):
        self.scale = (float(x), float(y), float(z))

    def reposition(self, x, y, z):
        self.translate = (float(x), float(y), float(z))

    def rotate_obj(self, x, y, z):
        self.rotate = (float(x), float(y), float(z))

    def draw(self):
        glPushMatrix()
        # Rotation
        # glRotatef(self.rotate[0], 1.0, 0.0, 0.0) #unused
        # glRotatef(self.rotate[2], 0.0, 0.0, 1.0) #unused
        #glTranslatef(0, 0, 0)

        # Translation
        glTranslatef(*self.translate)
        glRotatef(self.rotate[1], 0.0, 1.0, 0.0)

        # Scale
        glScalef(*self.scale)

        # Visualise
        visualization.draw(self.object)

        glPopMatrix()




import numpy as np
import pandas as pd

from murt import core, calculator

from murt.utils import objreader


class tracer:
    def __init__(self, scene_path=None):
        if scene_path is not None:
            vertices, triangles = objreader.read(scene_path)
            self.core = core.Tracer(vertices, triangles)
        else:
            self.core = None

    def load_scene(self, vertices, triangles):
        if self.core is not None:
            del self.core
        self.core = core.Tracer(vertices, triangles)

    def is_outdoor(self, pos):
        pos = np.array(pos)
        assert pos.shape[0] == 3
        pos = np.array(pos).astype('float32')
        return self.core.isOutdoor(pos) != 0

    def hit_nearest(self, fromPos, toPos):
        fromPos = np.array(fromPos)
        assert fromPos.shape[0] == 3
        toPos = np.array(toPos)
        assert toPos.shape[0] == 3
        fromPos = fromPos.astype('float32')
        toPos = toPos.astype('float32')
        distance = self.core.hitNearest(fromPos, toPos)
        return distance

    def trace(self, tx_pos, rx_pos):
        if self.core is None:
            return None
        tx_pos = np.array(tx_pos)
        rx_pos = np.array(rx_pos)
        assert rx_pos.shape[0] == 3
        assert tx_pos.shape[0] == 3
        tx_pos = tx_pos.astype('float32')
        rx_pos = rx_pos.astype('float32')
        result = self.core.trace(tx_pos, rx_pos)
        return result

    def get_depth_map(self, x_min, x_max, x_n, z_min, z_max, z_n):
        assert x_min < x_max
        assert z_min < z_max
        assert (x_n > 0) and (z_n > 0)

        upper_y = 1000
        below_y = -1000
        data_depth = []
        data_frame = None

        for current_x in np.linspace(x_min, x_max, x_n):
            for current_z in np.linspace(z_min, z_max, z_n):
                from_pos = [current_x, upper_y, current_z]
                to_pos = [current_x, below_y, current_z]
                distance = self.hit_nearest(from_pos, to_pos)
                # in case if there is no ground, skip
                if distance < 0:
                    depth = 0.0
                else:
                    depth = upper_y-distance
                data_row = {'x': current_x, 'z': current_z, 'depth': depth}
                # print(distance)
                data_depth.append(data_row)

        data_frame = pd.DataFrame(data_depth)
        return data_frame

    def get_traced_volume(self,
                          x_min, x_max, x_n,
                          y_min, y_max, y_n,
                          z_min, z_max, z_n,
                          tx_pos, tx_freq, mat_perm):
        # check input
        assert x_min < x_max and x_n > 0
        assert y_min < y_max and y_n > 0
        assert z_min < z_max and z_n > 0
        tx_pos = np.array(tx_pos)
        assert tx_pos.shape[0] == 3
        tx_pos = tx_pos.astype('float32')
        tx_freq = float(tx_freq)
        mat_perm = float(mat_perm)
        # trace
        results = self.core.traceVolume(x_min, x_max, x_n,
                                        y_min, y_max, y_n,
                                        z_min, z_max, z_n,
                                        tx_pos, tx_freq, mat_perm)
        # prepare data frame
        data_frame = None
        prepared_results = []
        assert results is not None
        for result in results:
            prepared_result = {'x': result[0],
                               'y': result[1],
                               'z': result[2],
                               'total_loss': result[3]}
            prepared_results.append(prepared_result)
        data_frame = pd.DataFrame(prepared_results)
        return data_frame

    @staticmethod
    def result_to_lines(results, tx_pos, rx_pos):
        lines = []
        for result in results:
            line = None
            if result[0] == 1:
                line = {'points': [tuple(tx_pos), tuple(rx_pos)],
                        'color': (0, 0.7, 0.3, 1)}
            if result[0] == 2:
                line = {'points': [tuple(tx_pos)], 'color': (0, 0, .9, 1)}
                for point in result[1]:
                    line['points'].append(tuple(point))
                line['points'].append(tuple(rx_pos))
            if result[0] == 3:
                line = {'points': [tuple(tx_pos), tuple(result[1]), tuple(rx_pos)],
                        'color': (0.7, 0.4, 0.7, 1)}
            if line is not None:
                lines.append(line)
        return lines

    @staticmethod
    def get_total_loss(tx_pos, rx_pos, results, tx_freq=2.4e9, material_permittivity=5.31):
        if results is None:
            return None

        losses = {'total_dB': None, 'signals': []}
        total_linear = 0
        for result in results:
            if result[0] == 1:
                loss_dB, delay = calculator.directLoss(tx_pos, rx_pos, tx_freq)
            elif result[0] == 2:
                edges = list(map(list, result[1]))
                loss_dB, delay = calculator.diffractLoss(
                    tx_pos, rx_pos, edges, tx_freq)
            elif result[0] == 3:
                refPos = list(result[1])
                loss_dB, delay = calculator.reflectLoss(tx_pos, rx_pos,
                                                        refPos, tx_freq,
                                                        material_permittivity)
            else:
                return None
            losses['signals'].append([loss_dB, delay])
            total_linear += np.power(10, -loss_dB/10)

        losses['total_dB'] = -10*np.log10(total_linear)
        return losses



from murt.window import Window
from murt import Tracer
from murt.utils.generator import SceneGenerator
from pyglet.window import key
from pyglet import text
import numpy as np


class RandomRunner(Window):
    def __init__(self, seed=9999):
        super().__init__()
        self.tracer = Tracer()
        self.generator = np.random.default_rng(seed)
        self.regenerate_scene()

    def on_key_release(self, pressed_key, modifier):

        if pressed_key == key.G:
            self.regenerate_scene()
        super().on_key_release(pressed_key, modifier)

    def regenerate_scene(self):
        scene_gen = SceneGenerator(self.generator.integers(0, 1000000))
        scene_gen.generate()
        self.scene = scene_gen.sceneComponents
        vertices, indice = scene_gen.get_triangles()
        self.tracer.load_scene(vertices, indice)
        self.lines_set = []
        for i in range(1):
            tx = (0.0, -3.0, 0.0)
            rx = (0.0, -3.0, 0.0)
            while self.tracer.is_outdoor(tx) is not True:
                tx = (self.generator.uniform(-150, 150),
                      self.generator.uniform(1.5, 2),
                      self.generator.uniform(-150, 150))

            while self.tracer.is_outdoor(rx) is not True:
                rx = (self.generator.uniform(-150, 150),
                      self.generator.uniform(2, 7),
                      self.generator.uniform(-150, 150))

            results = self.tracer.trace(tx, rx)
            lines = self.tracer.result_to_lines(results, tx, rx)
            self.lines_set += lines

    def render(self):
        super().render()



from setuptools import setup, Extension, find_packages
import numpy as np
import os


data_files = [
    ('murt-assets', [
        'assets/poznan.obj',
     'assets/components/cube.obj', 'assets/components/cube.mtl',
     'assets/components/house.obj', 'assets/components/house.mtl',
     'assets/components/ground.obj', 'assets/components/ground.mtl'])
]


def install():
    core_module = Extension('murt.core',
                            sources=['murt/core/core.cpp'],
                            include_dirs=[np.get_include()],
                            extra_compile_args=['-std=c++17'])

    calculator_module = Extension('murt.calculator',
                                  sources=['murt/core/calculator.cpp'],
                                  extra_compile_args=['-std=c++17'])
    setup(
        name="murt",
        version="0.0.4",
        author="Supawat Tamsri",
        author_email="contact@supawat.dev",
        description="Python Library for Multipath Ray Tracing",
        long_description=open('README.md').read(),
        long_description_content_type="text/markdown",
        license="MIT",
        keywords=['Telecommunications', 'Radio', 'Ray Tracing', 'Simulation'],
        packages=['murt', 'murt.utils', 'murt.apps',
                  'murt.engine', 'murt.window'],
        install_requires=['wheel', 'numpy', 'pyglet', 'pywavefront'],
        ext_modules=[core_module, calculator_module],
        data_files=data_files,
        include_package_data=True
    )


if __name__ == "__main__":
    install()



import numpy as np
import pandas as pd
import sys
import os
import logging
import pywavefront

from murt.window import Object
from murt import Tracer

COMPONENT_PATH = os.path.join(sys.prefix, "murt-assets")

pywavefront.configure_logging(
    logging.ERROR,
    formatter=logging.Formatter('%(name)s-%(levelname)s: %(message)s')
)


class SceneGenerator():
    def __init__(self, seed=0):
        self.cube_path = os.path.join(COMPONENT_PATH, "cube.obj")
        self.house_path = os.path.join(COMPONENT_PATH, "house.obj")
        self.ground_path = os.path.join(COMPONENT_PATH, "ground.obj")
        self.sceneComponents = []

        self.x_min = -140
        self.x_max = 140
        self.z_min = -140
        self.z_max = 140

        self.generator = np.random.default_rng(seed)
        self.globalVertices, self.globalIndice = None, None

    def generate(self):
        self.globalVertices = None
        self.globalIndice = None
        self.sceneComponents = []
        # Generate ground
        ground = Object(self.ground_path)
        self.sceneComponents.append(ground)
        # Generate buildings
        n_building = max(int(self.generator.normal(25, 10)), 2)
        n_building = min(n_building, 40)
        cube_percentage = 0.5
        for i in range(n_building):
            type_build = self.generator.random()
            if type_build < cube_percentage:
                building = self.generate_object(obj_path=self.cube_path,
                                                h_min=7, h_max=25, w_min=7,
                                                w_max=max(
                                                    self.generator.normal(20, 5), 15),
                                                d_min=7, d_max=30)
            else:
                building = self.generate_object(obj_path=self.house_path,
                                                h_min=5, h_max=20, w_min=7, w_max=15,
                                                d_min=7, d_max=20)
            self.sceneComponents.append(building)
        # Calculate triangles
        self.prepare_triangles()
        return

    def generate_object(self, obj_path=None, h_min=5,
                        h_max=30, w_min=7, w_max=30,
                        d_min=6, d_max=30):
        if obj_path is None:
            obj_path = self.cube_path

        building = Object(obj_path)

        # Generate Scale
        x_scale = self.generator.uniform(w_min,
                                         w_max)

        y_scale = self.generator.uniform(h_min,
                                         h_max)

        z_scale = self.generator.uniform(d_min,
                                         d_max)

        building.resize(x_scale, y_scale, z_scale)

        # Generate Position
        x = self.generator.uniform(self.x_min, self.x_max)
        z = self.generator.uniform(self.z_min, self.z_max)
        building.reposition(x, 0, z)

        # Generate Rotation

        if self.generator.random() < 0.5:
            rotate = 0
        else:
            rotate = self.generator.normal(90, 3)

        if self.generator.random() < 0.5:
            rotate += 90
        building.rotate_obj(0, rotate, 0)
        return building

    def get_scene(self):
        return self.sceneComponents

    def get_triangles(self):
        return self.globalVertices, self.globalIndice

    def prepare_triangles(self):
        self.globalVertices, self.globalIndice = None, None
        for component in self.sceneComponents:
            vertices, indexes = component.get_triangles()
            # scale
            vertices *= component.scale
            # rotate (yaw)
            angle = np.deg2rad(component.rotate[1])
            cos_angle = np.cos(-angle)
            sin_angle = np.sin(-angle)
            R_mat = np.array([[cos_angle, 0, sin_angle],
                              [0, 1, 0],
                              [-sin_angle, 0, cos_angle]])
            vertices = np.matmul(vertices, R_mat)
            # translate
            vertices += component.translate

            # add indice
            if self.globalIndice is None:
                self.globalIndice = np.array(indexes)
            else:
                indexes += self.globalVertices.shape[0]
                self.globalIndice = np.append(
                    self.globalIndice, indexes, axis=0)

            # add vertices
            if self.globalVertices is None:
                self.globalVertices = np.array(vertices)
            else:
                self.globalVertices = np.append(
                    self.globalVertices, vertices, axis=0)

    def get_depth_map(self, x_min, x_max, x_n, z_min, z_max, z_n):
        assert x_min < x_max
        assert z_min < z_max
        assert (x_n > 0) and (z_n > 0)

        current_tracer = Tracer()
        current_tracer.load_scene(*self.get_triangles())

        upper_y = 1000
        below_y = -1000
        data_depth = []
        data_frame = None

        for current_x in np.linspace(x_min, x_max, x_n):
            for current_z in np.linspace(z_min, z_max, z_n):
                from_pos = [current_x, upper_y, current_z]
                to_pos = [current_x, below_y, current_z]
                distance = current_tracer.hit_nearest(
                    from_pos, to_pos)
                # in case if there is no ground, skip
                if distance < 0:
                    depth = 0
                else:
                    depth = upper_y-distance
                data_row = {'x': current_x, 'z': current_z, 'depth': depth}
                # print(distance)
                data_depth.append(data_row)

        data_frame = pd.DataFrame(data_depth)
        return data_frame
